/*
 * Shows how reconnect works
 * Author: Catalin(ux) M. BOIE
 */

#define _GNU_SOURCE

#include <errno.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <time.h>
#include <arpa/inet.h>
#include <stdarg.h>

#include <Conn.h>

/* Global variables */
static unsigned short	debug = 20;
static FILE	*Logf = NULL;
static char	*log_file = "reconnect.log";


static void c_connected(struct Conn *C)
{
	Log(4, "%s: A connection was estabilished with id %llu)\n",
		__FUNCTION__, Conn_getid(C));
}

static void c_close(struct Conn *C)
{
	Log(5, "%s: id=%llu will close...\n",
		__FUNCTION__, Conn_getid(C));
}

static int c_data_cb(struct Conn *C, char *line)
{
	char *dump;

	if (debug >= 8) {
		dump = Conn_dump(line, strlen(line));
		Log(8, "data: recv: %s\n", dump);
		free(dump);
	}

	return 0;
}

static void c_data(struct Conn *C)
{
	Conn_for_every_line(C, c_data_cb);
}

static void c_error(struct Conn *C)
{
	Log(0, "%s id=%llu [%s]\n",
		__FUNCTION__, Conn_getid(C), Conn_strerror());
}

int main(void)
{
	int ret;
	struct Conn *C;

	Logf = fopen(log_file, "a+");
	if (!Logf) {
		fprintf(stderr, "Cannot open log file [%s] [%s]\n",
			log_file, strerror(errno));
		return 1;
	}
	if (debug > 0)
		setlinebuf(Logf);

	Conn_debug(Logf, debug);

	ret = Conn_init(0);
	if (ret == -1) {
		Log(0, "%s", Conn_strerror());
		return 1;
	}

	Conn_connected_cb = c_connected;
	Conn_data_cb = c_data;
	Conn_close_cb = c_close;
	Conn_error_cb = c_error;

	C = Conn_connect(PF_INET, SOCK_STREAM, "www.google.com", 80);
	if (C == NULL) {
		Log(0, "Error calling Conn_connect [%s]!\n",
			Conn_strerror());
		return 1;
	}

	Conn_set(C, CONN_PARA_AUTO_RECONNECT, 1);
	Conn_set(C, CONN_PARA_RECONNECT_DELAY, 10);

	while (1) {
		ret = Conn_poll(-1);
		if (ret == -1) {
			Log(0, "Error in poll [%s]!\n",
				Conn_strerror());
			break;
		}
	}

	Conn_shutdown();

	fclose(Logf);

	return 0;
}
